# -*- coding: utf-8 -*-
# This file is part of Lyric Grabber
# Lyric Grabber is not affiliated with azlyrics.com

'''
Lyric Grabber - download, display and save lyrics from the web
Copyright (C) 2014 Timothy Welborn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

contact:
welborntimothy@gmail.com

The full license can be found in "license.txt"
'''

from tkinter import *
from tkinter import filedialog
import os.path


class Window(object):

    def __init__(self, Handler, Widgets):

        self.Handler = Handler()
        self.Root = Tk()
        self.Root.title("Lyric Grabber")
        self.Root.resizable(0, 0)
        self.Lyrics = None

        # couldnt get icon to work any other way,
        # tried using multiple formats.
        Icon = PhotoImage(file="icon.png")
        self.Root.tk.call("wm", "iconphoto", self.Root, Icon)

        self.Widgets = Widgets()
        self.Widgets.Establish(self)

    def Loop(self):
        # center the window
        self.Root.update()
        winx = self.Root.winfo_width()
        winy = self.Root.winfo_height()
        scrx = self.Root.winfo_screenwidth()
        scry = self.Root.winfo_screenheight()
        self.Root.geometry(
            str(winx) + 'x' + str(winy) + '+' +
            str((scrx // 2) - (winx // 2)) + '+' +
            str((scry // 2) - (winy // 2)))

        self.ArtistEntry.focus()
        self.Root.bind("<Return>", self.Fetch)
        self.Root.bind("<Control-s>", self.SaveFile)
        self.Root.bind("<Control-r>", self.Reset)
        self.Root.mainloop()

    def ClearEntry(self, event=None):
        # clear entries
        self.Artist.set('')
        self.Song.set('')
        self.ArtistEntry.focus()

    def Fetch(self, event=None):

        Song = self.Song.get()
        Artist = self.Artist.get()

        # check for blanks
        if Artist == ' ' * len(Artist):
            self.StatusUpdate("Missing field: \"Artist\"")
            return(1)
        if Song == ' ' * len(Song):
            self.StatusUpdate("Missing field: \"Song\"")
            return(1)

        self.StatusUpdate("Grabbing lyrics")
        Lyrics = self.Handler.Main(Artist, Song)

        if Lyrics != 1:
            # show lyrics, clear entries, change title
            self.Reset()
            self.LyricST.insert(0.0, Lyrics)
            self.Root.title("Lyric Grabber | %s" % self.Handler.FileName)
            self.StatusUpdate()

        else:
            # debug
            self.StatusUpdate("Error occured, checking connection")
            Result = self.Handler.Main(Debug=True)
            if Result == 0:
                self.StatusUpdate("Invalid artist or song")
            elif Result == 1:
                self.StatusUpdate("Connection to host failed")

    def SaveFile(self, event=None):
        # ask to save
        self.StatusUpdate("Busy")
        AskSave = filedialog.asksaveasfile(
            mode='w',
            initialdir=os.path.expanduser('~'),
            initialfile=str(self.Handler.FileName + ".txt"),
            defaultextension=".txt")
        # save the file
        if AskSave:
            self.StatusUpdate("Saving lyrics")
            with open(AskSave.name, AskSave.mode) as LyricFile:
                LyricFile.write(self.LyricST.get(0.0, END))
                LyricFile.close()
        self.StatusUpdate()

    def Reset(self, event=None):
        # clear entries, reset title and status, refocus
        self.LyricST.delete(0.0, END)
        self.Root.title("Lyric Grabber")
        self.StatusUpdate()
        self.ClearEntry()

    def StatusUpdate(self, Status="Waiting"):
        # update status (Label)
        self.Status.configure(text=Status)
        self.Status.update()
